import geopandas as gpd
from shapely.geometry import Polygon, LineString, MultiLineString
import json
from server.map.add_lake import write_json_to_file


# Writes contents of a map json file to the response
def fetch_divided_map(self, file_name):
    self.send_response(200)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Extract contents from JSON file
    with open("server/lake_relations/" + file_name + "_div.json", "r") as file:
        data = file.read()

    # Write contents of the JSON file to response
    self.wfile.write(data.encode('utf-8'))


# Returns a list of [(sub_div_id, sub_div_center)]
def get_ids_and_centers(file_name):  # NB buggy
    # Expected format: [(id, [x,y]), (id, [x,y])]
    geo_data = gpd.read_file("server/lake_relations/" + file_name + "_div.json")
    subdivisions = []
    for index, row in geo_data.iterrows():
        sub_div_id = row['sub_div_id']
        sub_div_center = row['sub_div_center']

        print("sub_div_id: ", sub_div_id)

        subdivision = {
            'sub_div_id': sub_div_id,
            'sub_div_center': sub_div_center
        }
        subdivisions.append(subdivision)
    return subdivisions

# Create groups creates polygons which consist of groupings of related subdivisions
def create_groups(relation_file: str, data: list):
    try:
        print("Creating groups...")

        # Read lake relation from json file
        geo_data = gpd.read_file("server/lake_relations/" + relation_file + "_div.json")
        relation_data = geo_data[geo_data['geometry'].geom_type == 'Polygon']

        # Loop through each measurement and create groupings of subdivisions
        for measurement in data:
            subdiv_list = []

            for subdivision in measurement['Subdivisions']:
                subDivID = str(subdivision['SubdivID'])  # Convert to string to match format in feature
                group_id = subdivision['GroupID']  # Extract group ID

                # Find the matching subdivision in relation_data
                for index, feature in relation_data.iterrows():
                    # Add the new group ID to the correct subdivision
                    if feature['sub_div_id'] == subDivID:
                        subdiv_list.append((group_id, Polygon(feature['geometry'].exterior.coords)))
                        # subdiv_list.append((group_id, Polygon([feature['geometry']])))

            # Sort subdiv_list based on group_ids
            sorted_list = sorted(subdiv_list, key=lambda x: x[0])

            current_group = -1  # Current group_id
            new_shape = []      # List of subdivision geometries for current group

            # Merge subdivisions in a given group
            for element in sorted_list:
                # If the subdivision still belongs to the current group
                if element[0] == current_group:
                    new_shape.append(element[1])

                # New group id is found
                elif len(new_shape) > 1:
                    # Merger all subdivisions for previous group into a single shape
                    merged_polygon = MultiPolygon(new_shape).buffer(0)

                    # Convert to Polygon
                    if isinstance(merged_polygon, MultiPolygon):
                        merged_polygon = merged_polygon.convex_hull

                    # Structure the new polygon
                    merged_polygon_structure = {
                        "type": "Feature",
                        "properties": {
                            "group_id": current_group,
                        },
                        "geometry": {
                            "type": "Polygon",
                            "coordinates": [list(merged_polygon.exterior.coords)]
                        }
                    }

                    # Append new polygon to relation data
                    relation_data = relation_data.append(merged_polygon_structure, ignore_index=True)

                    # Update current group to new group_id and reset new_shape for next group
                    current_group = element[0]
                    new_shape = [element[1]]

        # Convert GeoDataFrame to JSON
        relation_data_json = json.loads(relation_data.to_json())

        # Write relation with group shapes to file
        write_json_to_file("server/lake_relations", "mjosa", relation_data_json)

    except Exception as e:
        print(f"Error in create_groups(): {e}")


